import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.media.j3d.*;

/**
 * Comportement personnalise du clavier :
 * Rotation du canon lorsqu'on presse les fleches droite-gauche du clavier
 */
public class CanonBehavior extends Behavior {

  // Groupe de transformation lie a la direction du canon
  private TransformGroup dirCanonTG = null;

  // Transformation associee au TransformGroup dirCanonTG
  private Transform3D rotation = new Transform3D();

  // Condition qui va declencher le stimulus
  private WakeupCondition wakeupCondition =
      new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);

  // Angle de la rotation du canon (degres)
  private float angle;

  /**
   * Constructeur de notre comportement lie au canon
   * @param dirCanonTG TransformGroup permettant de modifier la direction
   * du canon
   */
  public CanonBehavior(TransformGroup dirCanonTG) {
    this.dirCanonTG = dirCanonTG;
    angle = 0;
  }

  /**
   * Initialisation du comportement :
   * La methode processStimulus() sera appelee a chaque fois qu'une touche
   * du clavier sera pressee
   */
  public void initialize() {
    this.wakeupOn(wakeupCondition);
  }

  /**
   * Methode appelee a chaque fois que l'on appuie sur une touche du clavier
   * @param criteria critere ayant declenche le stimulus (ici un evenement
   * AWT correspondant a une pression sur une touche du clavier)
   */
  public void processStimulus(Enumeration criteria) {
    WakeupCriterion critere;
    AWTEvent[] events;
    KeyEvent evt;

    // On boucle sur les criteres ayant declenche le comportement
    while (criteria.hasMoreElements()) {
      // On recupere le premier critere de l'enumeration
      critere = (WakeupCriterion)criteria.nextElement();

      // On ne traite que les criteres correspondant a un evenement AWT
      if (critere instanceof WakeupOnAWTEvent) {
        // On recupere le tableau des evements AWT correspondant au critere
        events = ((WakeupOnAWTEvent)critere).getAWTEvent();

        if (events.length > 0) {
          // On recupere l'evement (le tableau d'evenements ne contient en
          // fait qu'un seul evenement)
          evt = (KeyEvent)events[events.length-1];

          // Traitement au cas par cas selon la touche du clavier pressee
          switch(evt.getKeyCode()) {
            // Fleche droite : rotation du canon vers la droite
            case KeyEvent.VK_RIGHT:
              angle = angle - 3;
              if (angle < -45)
                angle = -45;

              rotation.rotZ(angle * Math.PI / 180f);
              dirCanonTG.setTransform(rotation);
              break;

            // Fleche gauche : rotation du canon vers la gauche
            case KeyEvent.VK_LEFT:
              angle = angle + 3;
              if (angle > 45)
                angle = 45;

              rotation.rotZ(angle * Math.PI / 180f);
              dirCanonTG.setTransform(rotation);
              break;
          }  // fin switch(evt.getKeyCode())
        }  // fin if (events.length > 0)
      }  // fin if (wakeup instanceof WakeupOnAWTEvent)
    }  // fin while (criteria.hasMoreElements())

    // Une fois le stimulus traite, on reinitialise le comportement
    this.wakeupOn(wakeupCondition);
  }


}
