import java.util.*;

import javax.media.j3d.*;

/**
 * Comportement personnalise sur les collisions
 */
public class DetecteurCollision extends Behavior {

  protected Switch masqueSwitch = null;

  // Critere : un obus atteint le masque
  protected WakeupOnCollisionEntry entreZoneCollision = null;

  /**
   * Constructeur
   * @param node objet qui va subir la collision
   * @param appearance apparence de l'objet node
   * @param bounds zone d'influence du comportement collision
   */
  public DetecteurCollision(Node node,
                            Switch masqueSwitch,
                            Bounds bounds) {

    this.masqueSwitch = masqueSwitch;
    this.setSchedulingBounds(bounds);
    entreZoneCollision = new WakeupOnCollisionEntry(node);
  }

  /**
   * Initialise le comportement :
   * Le stimulus est declenche si un objet entre dans la zone de collision
   * de this.node, si il bouge dans la zone de collision ou si il sort de
   * la zone de collision.
   */
  public void initialize() {
    wakeupOn(entreZoneCollision);
  }

  /**
   * Le stimulus est traite ici.
   * @param criteria criteres ayant declenche le stimulus
   */
  public void processStimulus(Enumeration criteria) {

    // On boucle sur les criteres ayant declenche le comportement
    while (criteria.hasMoreElements()) {

      // On recupere le premier critere de l'enumeration
      WakeupCriterion theCriterion = (WakeupCriterion)criteria.nextElement();

      // Cas ou le critere correspond a l'entree dans la zone de collision
      if (theCriterion instanceof WakeupOnCollisionEntry) {
        masqueSwitch.setWhichChild(1);
      }
    }  // fin while (criteria.hasMoreElements())

    // Une fois le stimulus traite, on reinitialise le comportement
    wakeupOn(entreZoneCollision);
  }
}
