// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;
import java.net.*;
import java.io.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.loaders.objectfile.*;
import com.sun.j3d.loaders.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Jeu extends Applet {

  private static boolean isApplet = true ;

  // Switch contenant les 2 objets masque
  private Switch masqueSwitch = null;

  public Jeu() {
    if (!isApplet) {
      lanceApplication();
    }
  }

  public void lanceApplication() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D =
        new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    positionneCamera(simpleU);

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut
    // visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8 :
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  private void positionneCamera(SimpleUniverse su) {
    // Recuperation du groupe de transformation a partir de l'objet
    // SimpleUniverse afin de positionner la camera a Z = +7 metres puis de
    // tourner la camera d'un angle de +45 degres autour de X
    TransformGroup tg = su.getViewingPlatform().getViewPlatformTransform();

    // Translation + rotation
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0, 0, 10));
    Transform3D rotationX = new Transform3D();
    rotationX.rotX(45f * Math.PI/180f);
    Transform3D trans = new Transform3D();
    trans.mul(rotationX, translation);

    // On applique la transformation
    tg.setTransform(trans);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  private BranchGroup createSceneGraph() {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    // Construction du canon
    parent.addChild(construitCanon());

    // Construction du masque
    BranchGroup masque = construitMasque();
    parent.addChild(masque);

    // Creation de l'eclairage ambiant (toute la scene)
    Color3f ambLightColor = new Color3f(1.0f, 1.0f, 1.0f);
    AmbientLight ambLight = new AmbientLight(ambLightColor);
    ambLight.setInfluencingBounds(new BoundingSphere(new Point3d(), 100));
    parent.addChild(ambLight);

    // Creation de l'eclairage directionnel (toute la scene)
    Color3f dirLightColor = new Color3f(1.0f, 1.0f, 1.0f);
    Vector3f dirLightSceneDir  = new Vector3f(-1.0f, -1.0f, -1.0f);
    DirectionalLight dirLightScene =
        new DirectionalLight(dirLightColor,
                             dirLightSceneDir);
    dirLightScene.setInfluencingBounds(new BoundingSphere(new Point3d(), 100));
    parent.addChild(dirLightScene);

    // Creation de l'eclairage directionnel applique uniquement au masque
    Vector3f dirLightMasqueDir  = new Vector3f(0.0f, 1.0f, -1.0f);
    DirectionalLight dirLightMasque =
        new DirectionalLight(dirLightColor,
                             dirLightMasqueDir);
    dirLightMasque.setInfluencingBounds(new BoundingSphere(new Point3d(), 100));

    // Limitation de l'eclairage directionnel 2 au masque seulement
    dirLightMasque.addScope(masque);
    parent.addChild(dirLightMasque);

    return parent;
  }

  /**
   * Construction du canon
   * @return BranchGroup contenant tous les objets lies au canon
   */
  private BranchGroup construitCanon() {
    BranchGroup parent = new BranchGroup();

    // On deplace le canon de Y = -2.8
    TransformGroup parentTG = new TransformGroup();
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0, -2.8f, 0));
    parentTG.setTransform(translation);

    // Creation de l'apparence du canon
    Appearance appCanon = new Appearance();
    Color3f ambientColor = new Color3f(0.5f, 0.5f, 0.0f);
    Color3f emissiveColor = new Color3f(0.0f, 0.0f, 0.0f);
    Color3f diffuseColor = new Color3f(0.5f, 0.5f, 1.0f);
    Color3f specularColor = new Color3f(1.0f, 1.0f, 1.0f);
    Material material = new Material(ambientColor,
                                     emissiveColor,
                                     diffuseColor,
                                     specularColor,
                                     64);
    appCanon.setMaterial(material);

    // Creation des deux parties du canon : le corps et le tube
    Box corps = new Box(0.35f, 0.3f, 0.2f, appCanon);
    Cylinder tube = new Cylinder(0.08f, 0.8f, appCanon);
    TransformGroup transTubeTG = new TransformGroup();
    Transform3D transTube = new Transform3D();
    transTube.setTranslation(new Vector3f(0, (float)(0.3/2 + 0.8/2), 0));
    transTubeTG.setTransform(transTube);
    transTubeTG.addChild(tube);

    // Ajout des deux objet au BranchGroup parent
    parentTG.addChild(transTubeTG);
    parentTG.addChild(corps);
    parent.addChild(parentTG);

    return parent;
  }

  /**
   * Chargement du masque
   * @return BranchGroup
   */
  private BranchGroup construitMasque() {
    BranchGroup parent = new BranchGroup();

    // Initialisation du switch qui va contenir les 2 masques.
    // Par defaut, c'est le masque intact (objet numero 0) qui est affiche
    masqueSwitch = new Switch(0);
    masqueSwitch.setCapability(Switch.ALLOW_SWITCH_WRITE);

    // On reduit la taille du masque et on le deplace de Y = +5.0
    TransformGroup parentTG = new TransformGroup();
    Transform3D scale = new Transform3D();
    scale.setScale(0.5);
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0, 5.0f, 0));
    Transform3D transScale = new Transform3D();
    transScale.mul(translation ,scale);
    parentTG.setTransform(transScale);

    // Chargement des deux masques intact et touche (objets Wavefront)
    BranchGroup masqueIntact = loadWavefrontObject("masqueintact.obj");
    BranchGroup masqueTouche = loadWavefrontObject("masquetouche.obj");

    // L'objet masqueTouche est renverse (tete en bas) par rapport au masque
    // intact
    TransformGroup masqueToucheRotTG = new TransformGroup();
    Transform3D rotationZ = new Transform3D();
    rotationZ.rotZ(Math.PI);
    masqueToucheRotTG.setTransform(rotationZ);
    masqueToucheRotTG.addChild(masqueTouche);

    // Ajout des 2 objets au switch
    masqueSwitch.addChild(masqueIntact);
    masqueSwitch.addChild(masqueToucheRotTG);

    // Creation du groupe de transformation lie au billboard
    TransformGroup billboardTG = new TransformGroup();
    billboardTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    billboardTG.addChild(masqueSwitch);
    parentTG.addChild(billboardTG);

    // Creation et activation du billboard
    Billboard billboard =
            new Billboard(billboardTG,
                          Billboard.ROTATE_ABOUT_AXIS,
                          new Vector3f(1.0f, 0.0f, 0.0f));
    billboard.setSchedulingBounds(new BoundingSphere());

    // Ajout du masque et du billboard a la scene 3D
    parentTG.addChild(billboard);
    parent.addChild(parentTG);

    return parent;
  }

  /**
   * Chargement de l'objet Wavefront (masque.obj) ainsi que les materiaux qui
   * lui sont associes
   * @param filename nom du fichier de l'objet a charger
   * @return BranchGroup branch group contenant l'objet Wavefront
   */
  private BranchGroup loadWavefrontObject(String filename) {
    ObjectFile waveFrontObject = new ObjectFile(ObjectFile.RESIZE);
    Scene scene = null;
    int error = 0;

    try {
      if (!isApplet) {
        scene = waveFrontObject.load(filename);
      }
      else {
        URL url = this.getCodeBase();
        String path = url.toString() + filename;
        try {
          URL fileUrl = new URL(path);
          scene = waveFrontObject.load(fileUrl);
        }
        catch (MalformedURLException ex2) {
          System.out.println("Impossible de charger le fichier : " +
                             filename);
          error = -1;
        }
      }  // fin else
    }  // fin try
    catch (ParsingErrorException ex) {
      System.out.println("Fichier defectueux");
      error = -1;
    }
    catch (IncorrectFormatException ex) {
      System.out.println("Mauvais format de fichier");
      error = -1;
    }
    catch (FileNotFoundException ex) {
      System.out.println("Fichier " + filename + " non trouve");
      error = -1;
    }

    if (error != 0)
      System.exit(error);

    return scene.getSceneGroup();
  }

  public void init() {
    if (isApplet) {
      lanceApplication();
    }
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments de la ligne de commande
   */
  public static void main(String[] args) {
    isApplet = false ;
    Frame frame = new MainFrame(new Jeu(), 640, 480);
  }
}
