// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Jeu extends Applet {

  public Jeu() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D =
        new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    positionneCamera(simpleU);

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D que l'on veut
    // visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8 :
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  public void positionneCamera(SimpleUniverse su) {
    // Recuperation du groupe de transformation a partir de l'objet
    // SimpleUniverse afin de positionner la camera a Z = +10 metres puis de
    // tourner la camera d'un angle de +45 degres autour de X
    TransformGroup tg = su.getViewingPlatform().getViewPlatformTransform();

    // Translation + rotation
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0, 0, 10));
    Transform3D rotationX = new Transform3D();
    rotationX.rotX(45f * Math.PI/180f);
    Transform3D trans = new Transform3D();
    trans.mul(rotationX, translation);

    // On applique la transformation
    tg.setTransform(trans);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {
    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    // Construction du canon
    parent.addChild(construitCanon());

    return parent;
  }

  /**
   * Construction du canon
   * @return BranchGroup contenant tous les objets lies au canon
   */
  public BranchGroup construitCanon() {
    BranchGroup parent = new BranchGroup();

    // On deplace le canon de Y = -2.8
    TransformGroup parentTG = new TransformGroup();
    Transform3D translation = new Transform3D();
    translation.setTranslation(new Vector3f(0, -2.8f, 0));
    parentTG.setTransform(translation);

    Appearance appCanon = new Appearance();

    // Creation des deux parties du canon : le corps et le tube
    Box corps = new Box(0.35f, 0.3f, 0.2f, appCanon);
    Cylinder tube = new Cylinder(0.08f, 0.8f, appCanon);
    TransformGroup transTubeTG = new TransformGroup();
    Transform3D transTube = new Transform3D();
    transTube.setTranslation(new Vector3f(0, (float)(0.3/2 + 0.8/2), 0));
    transTubeTG.setTransform(transTube);
    transTubeTG.addChild(tube);

    // Ajout des deux objet au BranchGroup parent
    parentTG.addChild(transTubeTG);
    parentTG.addChild(corps);
    parent.addChild(parentTG);

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments de la ligne de commande
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(new Jeu(), 640, 480);
  }
}
